import wave
import io
import requests


def url_audio_length(url):
    """Get audio length in seconds from URL

    Args:
        url (str): Audio URL

    Returns:
        int: Audio length in seconds
    """
    r = requests.get(url)
    infofile = wave.open(io.BytesIO(r.content), 'r')
    frames = infofile.getnframes()
    rate = infofile.getframerate()
    duration = frames / float(rate)
    return duration



import json


def read_jsonl(filepath):
    """Reads JSONL and outputs JSON

    Args:
        filepath (str): Filepath to JSONL file

    Returns:
        JSON: JSON file
    """
    data = [json.loads(l) for l in open(filepath)]
    return data


def write_jsonl(injson, filepath):
    """Writes JSONL

    Args:
        injson (JSON): JSON file
        filepath (str): Output filepath (include file name and extension)
    """
    with open(filepath, 'w', encoding='utf-8') as f:
        for item in injson:
            f.write(json.dumps(item) + "\n")


def read_json(filepath):
    """Read JSON file

    Args:
        filepath (str): Filepath to JSON file

    Returns:
        JSON: JSON file
    """
    with open(filepath) as f:
        return json.load(f)


def write_json(injson, filepath):
    """Writes JSON

    Args:
        injson (JSON): JSON file
        filepath (str): Output filepath (include file name and extension)
    """
    with open(filepath, 'w', encoding='utf-8') as f:
        json.dump(injson, f, ensure_ascii=False)



import glog as log
import os


def create_new_folder(new_folder_name, directory=None):
    """Creates folder in directory

    Args:
        new_folder_name (str): folder name
        directory (str, optional): Directory. Defaults to None.

    Returns:
        str: New directory
    """
    if directory:
        filedir = directory
    else:
        filedir = os.getcwd()
    new_dir = os.path.join(filedir, new_folder_name)
    if not os.path.isdir(new_dir):
        os.makedirs(new_dir)
    return new_dir


def out_filepath(filepath, new_directory='', new_prefix='', new_suffix='', new_filetype='', new_filename=''):
    """Updates input filepath to output filepath.

    Args:
        filepath (str): Input filepath (include file name and extension)
        new_directory (str, optional): Path to new directory. Defaults to ''.
        new_prefix (str, optional): New file prefix. Defaults to ''.
        new_suffix (str, optional): New file suffix. Defaults to ''.
        new_filetype (str, optional): New file extension/filetype, if none then retains input file type. Defaults to ''.

    Returns:
        [str]: Output file path
    """
    directory, filename = os.path.split(filepath)
    head, ext = os.path.splitext(filename)
    if new_filename:
        head = new_filename
    if new_filetype:
        updated_filename = f'{new_prefix}{head}{new_suffix}.{new_filetype}'
    else:
        updated_filename = f'{new_prefix}{head}{new_suffix}{ext}'
    if new_directory:
        new_outfilepath = os.path.join(new_directory, updated_filename)
    else:
        new_outfilepath = os.path.join(directory, updated_filename)
    return new_outfilepath


def filesize(filepath):
    """Get file size. Output in bytes.

    Args:
        file (str): File path

    Returns:
        [str]: File size
    """

    def sizeof_fmt(num, suffix='B'):
        for unit in ['', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi']:
            if abs(num) < 1024.0:
                return "%3.1f%s%s" % (num, unit, suffix)
            num /= 1000.0
        return "%.1f%s%s" % (num, 'Yi', suffix)

    return sizeof_fmt(os.path.getsize(filepath))



import pandas as pd
import re


def custompivot(in_info):
    """Summarize series or dataframe. Returns count and percent distribution

    Args:
        in_info (Series or DataFrame): [description]

    Returns:
        [type]: [description]
    """
    if type(in_info) == pd.core.series.Series:
        if (type(in_info.iloc[0]) == int or type(in_info.iloc[0]) == float) and len(in_info.index) > 10:
            counts = in_info.value_counts(dropna=False, bins=10)
        else:
            counts = in_info.value_counts(dropna=False)
    elif type(in_info) == pd.core.frame.DataFrame:
        counts = in_info.groupby(list(in_info.columns)).size()
    else:
        counts = 0
    percent = counts / counts.sum()
    fmt = '{:.2%}'.format
    output = pd.DataFrame({'counts': counts, 'percent': percent.map(fmt)})
    return output


def check_column(df, col, bins=4):
    """Provides some metrics and distribution info with the column provided

    Args:
        df (Pandas Dataframe): Input pandas dataframe
        col (str): Dataframe column name that should be analyzed
        bins (int): Number of bins to calculate column distribution. Defaults to 4 bins.

    Returns:
        dict: column length, null value count, duplicate value count, word count distribution (if column contains mostly strings), numeric distribution (if column contains mostly numeric data), and the first 3 values
    """
    len_df = len(df.index)
    not_null_column = [str(x) for x in list(df[col]) if str(x).lower() != 'nan']
    not_null_count = len(not_null_column)
    duplicate_values = df[col].duplicated().any()
    unique_values_count = df[col].nunique()
    null_values_count = df[col].isna().sum()
    outdict = {
        'column_header_name': col,
        'len_df': len_df,
        'null_values_count': null_values_count,
        'null_values_percent': round(null_values_count / len_df * 100, 2),
        'not_null_values_count': not_null_count,
        'duplicate_values_bool': duplicate_values,
        'unique_values_count': unique_values_count,
    }
    try:
        if pd.api.types.is_string_dtype(df[col]):
            df['__word_count'] = df[col].apply(lambda x: len(re.findall(r'\w+', x)) if pd.notna(x) else 0)
            distro_count = pd.Series(pd.cut(df['__word_count'], bins=bins, precision=0)).value_counts().to_frame().reset_index()
            distro_col_name = 'word_count_distribution_percent'
            col_type = 'string'
        elif pd.api.types.is_numeric_dtype(df[col]):
            distro_count = pd.Series(pd.cut(df[col], bins=bins, precision=0)).value_counts().to_frame().reset_index()
            distro_col_name = 'count_distribution_percent'
            col_type = 'numeric'
        else:
            pass
        distro_count.rename(columns={'index': 'range'}, inplace=True)
        distro_count['range'] = distro_count['range'].astype(str)
        distro_count = distro_count.to_dict('records')
        for d in distro_count:
            if col_type == 'string':
                d['__word_count'] = round(d['__word_count'] / len_df * 100, 2)
            else:
                d[col] = round(d[col] / len_df * 100, 2)
        outdict.update({distro_col_name: distro_count})
    except:
        pass

    try:
        outdict.update({'first_3_values': '|'.join(not_null_column[:3])})
    except:
        pass
    return outdict


def df_check_columns(df):
    result = []
    for col in df.columns:
        result.append(check_column(df, col))
    result_df = pd.DataFrame(result)
    return result_df



import random


def hex_to_rgb(hex):
    hex = hex.lstrip('#')
    hlen = len(hex)
    hlen_div = int(hlen / 3)
    return tuple(int(hex[i : i + hlen_div], 16) for i in range(0, hlen, hlen_div))


def rgb_to_hex(rgb):
    return '#%02x%02x%02x' % rgb


def random_hex():
    r = lambda: random.randint(0, 255)
    x = '#%02X%02X%02X' % (r(), r(), r())
    return x



import datetime
import timeit
import glog as log
import os
from pytz import timezone
import dateutil.parser


def time_start():
    return timeit.default_timer()


def timer(time_start, length=1):
    """Returns time elapsed time in minutes and time per unit.

    Args:
        time_start (timeit.default_timer()): Start time in timeit.default_timer()
        length (int): [description]
    """
    time_end = timeit.default_timer()
    elapsed_minutes = round((time_end - time_start) / 60, 2)
    elapsed_rate_seconds = round((time_end - time_start) / length, 2)
    log.info(f'--- Total Running Time: {elapsed_minutes} minutes, on average {elapsed_rate_seconds} seconds per unit.')
    return (elapsed_minutes, elapsed_rate_seconds)


def convert_timezone(intime, timezone_origin, timezone_new):
    """Convert time to specified timezone

    Args:
        intime ([type]): [description]
        timezone_origin ([type]): [description]
        timezone_new ([type]): [description]

    Returns:
        [type]: [description]
    """
    dt = dateutil.parser.parse(intime)
    ct = timezone(timezone_origin).localize(dt)
    new_time = ct.astimezone(timezone(timezone_new))
    outtime = new_time.isoformat()
    # str(new_time.strftime("%m/%d/%Y %H:%M:%S"))
    return outtime



from concurrent.futures import ThreadPoolExecutor, as_completed
import glog as log


def threadhandler(infunc, list_items, max_workers=None):
    """Function applies function parallely, `infunc`, to list of items

    Args:
        infunc (function): Function that takes in 1 or 2 inputs, and outputs
        list_items (list): List of items. Can be tuple [(x,y),(a,b)]
        max_workers (int, optional): Number of workers, or number of threads you want to initiate to parallelize the task. Defaults to None.

    Returns:
        [type]: [description]
    """
    log(f"-- Initiating thread for processing {len(list_items)} items with {infunc.__name__} function")
    list_results = []
    with ThreadPoolExecutor(max_workers=max_workers) as executor:
        # If items within list are tuples to unpack. If function requires more than 1 argument.
        if all([isinstance(x, tuple) for x in list_items]):
            future_to_row = {executor.submit(infunc, item[0], item[1]): item for item in list_items}
        else:
            future_to_row = {executor.submit(infunc, item): item for item in list_items}
        for future in as_completed(future_to_row):
            results = future.result()
            list_results.append(results)
    return list_results



import secrets
import string
import random


def generate_random_string(string_length=None, words=1):
    gen_words = []
    for word in range(words):
        string_length = random.randint(2, 12) if string_length is None else string_length
        rando_word = ''.join(secrets.choice(string.ascii_uppercase + string.ascii_lowercase) for i in range(string_length))
        gen_words.append(rando_word)
        string_length = None
    return ' '.join(gen_words)



from setuptools import setup, find_packages

setup(
    name='haytool',
    version='0.0.7',
    description="Hayman's toolbox",
    long_description_content_type='text/markdown',
    author='Hayman',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    packages=find_packages(),
    python_requires='>=3.6, <4',
    install_requires=['boto3', 'glog', 'wave', 'pandas', 'pytz', 'requests'],
)
